@@ -15,7 +15,7 @@ eval {
 	require GD::Graph::bars; 
 	$gd_ok = 1;
 };
-my $VERSION = '1.20';
+my $VERSION = 1.22;
 
 print "\n This script will plot histograms of value frequencies\n\n";
 
@@ -150,9 +150,9 @@ unless (defined $y_ticks) {
 ####### Main ###########
 
 ### Load the file
-print " Loading data from file $infile....\n";
 my $Data = Bio::ToolBox::Data->new(file => $infile) or
-	die " No data loaded!\n";
+	die " Unable to load data file!\n";
+printf " Loaded %s features from $infile.\n", format_with_commas( $Data->last_row );
 
 # load the dataset names into hashes
 my (%dataset_by_name, %dataset_by_id); # hashes for name and id
@@ -160,9 +160,13 @@ my $i = 0;
 foreach my $name ($Data->list_columns) {
 	# check column header names for gene or window attribute information
 	# these won't be used for graph generation, so we'll skip them
-	next if $name =~ /^(?:name|id|class|type|alias|probe|chr|
+	if ($name =~ /^(?:name|id|class|type|alias|probe|chr|
 		chromo|chromosome|seq|sequence|refseq|contig|scaffold|start|stop|end|mid|
-		midpoint|strand|primary_id)$/xi;
+		midpoint|strand|primary_id)$/xi
+	) {
+		$i++;
+		next;
+	}
 	
 	# record the data set name
 	$dataset_by_id{$i} = $name;
